
proc getColorMask { str var} {
    set intMask [ scan $str "%d"]
    set ${var}(red) 0
    set ${var}(green) 0
    set ${var}(blue) 0
    set ${var}(alpha) 0
    if { $intMask >= 1000} {
	set ${var}(red) 1
	incr intMask -1000
    }
    if { $intMask >= 100} {
	set ${var}(green) 1
	incr intMask -100
    }
    if { $intMask >= 10} {
	set ${var}(blue) 1
	incr intMask -10
    }
    if { $intMask >= 1} {
	set ${var}(alpha) 1
	incr intMask -1
    }
}

proc strColorMask { var} {
    set msk ""
    foreach idx {red green blue alpha} {
	if { [ subst $${var}($idx)]} {
	    append msk "1"
	} else {
	    append msk "0"
	}
    }
    return $msk
}

proc StereoGetInfo { } {
    array set ::GID_Stereo [ GiD_Info stereo]
    # copy variables to check user changes
    foreach idx [ list Stereo StereoMode EyeDistance FlipEyes \
		      RelativeEyeDistance LeftEyeColorMask RightEyeColorMask] {
	set ::GID_Stereo(Old$idx) $::GID_Stereo($idx)
    }
    getColorMask $::GID_Stereo(LeftEyeColorMask) ::GID_Stereo_Left_Mask
    getColorMask $::GID_Stereo(RightEyeColorMask) ::GID_Stereo_Right_Mask
    if { ![ info exists ::GID_Stereo(DynamicApply)]} {
	set ::GID_Stereo(DynamicApply) 0
    }
    
    # NORMAL o PERSPECTIVE
    set ::GID_Stereo(ModoPerspectiva) [ GiD_Info perspectivemode ]
    set ::GID_Stereo(OldModoPerspectiva) $::GID_Stereo(ModoPerspectiva)
}

proc CreateLabelMenuButtonFrame { w txt lst_lbl lst_val var} {
    frame $w -bd 0
    label $w.lbl -text $txt
    TTKMenubutton $w.mb -labels $lst_lbl -values $lst_val -textvariable $var
    grid $w.lbl $w.mb -sticky nes
    grid configure  $w.mb -sticky nw
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
}

proc CreateLabelMenuButton { w txt lst_lbl lst_val var} {
    label ${w}_lbl -text $txt
    TTKMenubutton ${w}_mb -labels $lst_lbl -values $lst_val -textvariable $var \
	-modifycmd "StereoDynamicApply $w ;\#"
    # grid ${w}_lbl ${w}_mb -sticky nes
    # grid configure  ${w}_mb -sticky nw
    # grid rowconfigure $w 0 -weight 1
    # grid columnconfigure $w 0 -weight 1
}

proc CreateColorMaskCB { w var} {
    frame $w
    foreach idx {red green blue alpha} lbl [list [_ "red"] [_ "green"] [_ "blue"] [_ "alpha"]] {
	ttk::checkbutton ${w}.$idx -text $lbl -variable ${var}($idx) \
	-command "StereoDynamicApply $w ;\#"
    }
    grid ${w}.red ${w}.green ${w}.blue ${w}.alpha -padx 4
}

proc StereoApply { w } {
    if { $::GID_Stereo(OldModoPerspectiva) != $::GID_Stereo(ModoPerspectiva)} {
	GiD_Process Projection $::GID_Stereo(ModoPerspectiva)
	set ::GID_Stereo(OldModoPerspectiva) $::GID_Stereo(ModoPerspectiva)
    }
    set ::GID_Stereo(LeftEyeColorMask) [ strColorMask ::GID_Stereo_Left_Mask]
    set ::GID_Stereo(RightEyeColorMask) [ strColorMask ::GID_Stereo_Right_Mask]
    foreach idx [ list EyeDistance FlipEyes \
		      RelativeEyeDistance LeftEyeColorMask RightEyeColorMask] {
	if { $::GID_Stereo(Old$idx) != $::GID_Stereo($idx)} {
	    GiD_Process View Stereo $idx $::GID_Stereo($idx) escape escape
	    # WarnWinText "GiD_Process View Stereo $idx $::GID_Stereo($idx) escape escape"
	    set ::GID_Stereo(Old$idx) $::GID_Stereo($idx)
	}
    }
    if { $::GID_Stereo(OldStereoMode) != $::GID_Stereo(StereoMode)} {
	GiD_Process View Stereo $::GID_Stereo(StereoMode) escape escape
	# WarnWinText "GiD_Process View Stereo $val escape escape"
	set ::GID_Stereo(OldStereo) $::GID_Stereo(Stereo)
    }
    if { $::GID_Stereo(OldStereo) != $::GID_Stereo(Stereo)} {
	set val "No"
	if { $::GID_Stereo(Stereo)} {
	    set val "Yes"
	}
	GiD_Process View Stereo $val escape escape
	# WarnWinText "GiD_Process View Stereo $val escape escape"
	set ::GID_Stereo(OldStereo) $::GID_Stereo(Stereo)
    }
}

proc StereoDynamicApply { w} {
    if { $::GID_Stereo(DynamicApply)} {
	StereoApply $w
    }
}

proc StereoClose { w } {
    destroy $w
}

proc StereoW { { w .gid.wStereoW} } {
    
    InitWindow $w [_ "Stereo settings"] PrePostStereoWindowGeom StereoW
    if { ![ winfo exists $w] } return ;# windows disabled || usemorewindows == 0
    #toplevel $w

    StereoGetInfo

    set top $w.top
    set but $w.but
    CreateApplyCloseDecorationWindow $w $top $but "StereoApply $w" "StereoClose $w"
    # CreateDecorationWindow $w $top $but \
    # 	[ list [_ "Apply"] [_ "Close"]] \
    # 	[ list "StereoApply $w" "StereoClose $w"] \
    # 	[ list <Return> <Escape>]

    label $top.lbl -text [_ "Stereo settings"]:

    ttk::checkbutton $top.useStereo -variable ::GID_Stereo(Stereo) -text [_ "Use stereo"] \
	-command "StereoDynamicApply $w ;\#"

    frame $top.f
    ttk::checkbutton $top.f.perspCB -text [_ "Perspective"] \
	-variable ::GID_Stereo(ModoPerspectiva) \
	-onvalue PERSPECTIVE -offvalue NORMAL \
	-command "StereoDynamicApply $w ;\#"
    ttk::button $top.f.perspOpt -text "-->" -command PerspectiveW -width 5

    grid $top.f.perspCB $top.f.perspOpt -sticky en

    CreateLabelMenuButton $top.stereo_mode [_ "stereo mode"]: \
	[ list [_ "Anaglyph colors"] [_ "Quad buffers"]] \
	[ list AnaglyphColors QuadBuffers] \
	::GID_Stereo(StereoMode)

    #scale ....
    label $top.eye_dist_lbl -text [_ "eye distance"]:
    scale $top.eye_dist_sc -from 0.0 -to 0.1 -resolution 0.005 \
	-variable ::GID_Stereo(EyeDistance) -orient horizontal \
	-command "StereoDynamicApply $w ;#"

    ttk::checkbutton $top.flipEyes -variable ::GID_Stereo(FlipEyes) -text [_ "switch eyes"] \
	-command "StereoDynamicApply $w ;\#"

    CreateLabelMenuButton $top.relative_distance [_ "eye distance mode"]: \
	[ list [_ "scaled with the model"] [_ "absolute eye distance"]] \
	[ list 1 0] \
	::GID_Stereo(RelativeEyeDistance)

    label $top.left_mask_lbl -text [_ "left color mask"]:
    CreateColorMaskCB $top.left_mask_cb ::GID_Stereo_Left_Mask

    label $top.right_mask_lbl -text [_ "right color mask"]:
    CreateColorMaskCB $top.right_mask_cb ::GID_Stereo_Right_Mask


    ttk::checkbutton $top.dynamicUpdate -variable ::GID_Stereo(DynamicApply) \
	-text [_ "Dynamic update"]

    # empaquetamiento # -columnspan 2
    grid $top.useStereo $top.f -sticky w -padx 8 -pady 4
    grid configure $top.f -sticky e 
    # grid $top.lbl -sticky w -columnspan 2
    #grid $top.stereo_mode -sticky w -padx 8
    grid $top.stereo_mode_lbl $top.stereo_mode_mb -sticky nes
    grid configure $top.stereo_mode_mb -sticky nw

    grid $top.eye_dist_lbl $top.eye_dist_sc -sticky nes
    grid configure $top.eye_dist_sc -sticky nwe

    grid $top.flipEyes -sticky w -row 3 -column 1
    # grid $top.relative_distance -sticky w -padx 8
    grid $top.relative_distance_lbl $top.relative_distance_mb -sticky nes
    grid configure $top.relative_distance_mb -sticky nw

    grid $top.left_mask_lbl $top.left_mask_cb -sticky nes
    grid configure $top.left_mask_cb -sticky nw

    grid $top.right_mask_lbl $top.right_mask_cb -sticky nes
    grid configure $top.right_mask_cb -sticky nw

    grid $top.dynamicUpdate -sticky w -columnspan 2 -padx 8 -pady 8
}