
proc WarnWinDirect { text { parent .gid} { icn question}} {
    if { ( $parent != "") && [ winfo exists $parent]} {
	tk_messageBox -message $text -parent $parent \
	    -title Warning -icon $icn -type ok
    } else {
	tk_messageBox -message $text \
	    -title Warning -icon $icn -type ok
    }
}

set GIDDEFAULT $env(GIDDEFAULT)
set GIDDEFAULTTCL $GIDDEFAULT
catch { set GIDDEFAULTTCL $env(GIDDEFAULTTCL) }

#----------------------Configuration section-----------------------------

# configuration is now in a separated file

source [file join $GIDDEFAULTTCL ConfigureProgram.tcl]

#----------------------End configuration section-------------------------



if { $GIDDEFAULT == "" } {
    puts [_ " (in tclfile) to run 'gid is necessary to set the environment variable: GIDDEFAULT to the path of the executables"]
    destroy .
}


set winole 0
catch { set winole [ gidolewin]}
if { $winole == "0" } {
    toplevel .gid
} else {
    toplevel .gid -use $winole
}
wm withdraw .

if { ( $::tcl_platform(os) == "Darwin")} {
    catch {
	console hide
    }
}

# RAMSAN: it is important not to have external dependencies
#IsSplashWindowOnInDefault esta definido en tclfileP.tcl
# y hay que leer el tclIndex
set auto_path [list $GIDDEFAULTTCL [file join $GIDDEFAULTTCL tk]]

proc GiDSplash { } {
    global GidPriv

    set mainversion 9 ;#GiD_Info GiDVersion can't be used because the togl object .central.s does not exist
    set txt [concat [_ "Version"] $mainversion]

    set w .splash
    if { [winfo exists $w] } { destroy $w }
    toplevel $w
    set im [image create photo -file $GidPriv(SplashWindow)]
    set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
    set y [expr [winfo screenheight .splash]/2-[image height $im]/2]

    wm withdraw .gid

    wm geom $w +$x+$y
    #wm transient $w  .gid
    wm overrideredirect .splash 1
    set base $w.l
    label $base -image $im -relief ridge -bd 2

    #draw version txt over the image
    label $base.lv -text $txt -font "Helvetica 20 bold italic" -fg #aaaaaa -bg black -bd 0 -highlightthickness 0
    place $base.lv -x 220 -y 410 -in $base

    pack $base
    update
    bind $w <1> "destroy $w"

    after 2000 "if { [ winfo exists $w] } { destroy $w }"
}

proc InitialOpenGLConfiguration {} {
    wm withdraw .gid
    
    set ::UseLocalOpenGL $::GidPriv(SoftwareOpenGL)
    
    toplevel .softogl -class OverlayWindow
    wm title .softogl [_ "Initial graphics configuration ( OpenGL)"]
    set sx [ winfo screenwidth .]
    set sy [ winfo screenheight .]
    wm geometry .softogl +[ expr ( $sx - 300) / 2]+[ expr ( $sy - 200) / 2]
    
    set def_back [ .softogl cget -background]
    
    ttk::frame .softogl.f  -relief groove
    # -bd 2 -relief groove
    
    set base .softogl.f
    ttk::label $base.lbl -text [_ "Initial graphics configuration ( OpenGL)"]:
    ttk::radiobutton $base.rd1 \
	-text [_ "Fast visualization mode."] \
	-value 0 -variable ::UseLocalOpenGL
    text $base.rd1txt -wrap word -width 40 -height 3 -relief flat
    #-bg $def_back
    $base.rd1txt ins end \
	[_ "( Models will be drawn quicker but some issues could appear. I'm confident about my Graphic Card and I want to use its capabilities)."]
    $base.rd1txt configure -state disabled
    
    ttk::radiobutton $base.rd2 \
	-text [_ "Safe visualization mode."] \
	-value 1 -variable ::UseLocalOpenGL
    text $base.rd2txt -wrap word -width 40 -height 3 -relief flat
    #-bg $def_back
    $base.rd2txt ins end \
	[_ "( Models will be drawn slower but the artifacts would disappear. I'm experiencing troubles with my Graphic Card and I want to use OpenGL by software)."]
    $base.rd2txt configure -state disabled
    
    text .softogl.f.txt -wrap word -width 40 -height 3 -relief flat
    #-bg $def_back
    .softogl.f.txt ins end "( This preference can always be accessed through the 'Graphical' panel in the 'Utilities-->Preferences' window, or writting 'gid -openglconfig' in the command line.)"
    .softogl.f.txt configure -state disabled
    
    ttk::button .softogl.b1 -text Ok -command {
	if { $::GidPriv(SoftwareOpenGL) != $::UseLocalOpenGL} {
	    set ::GidPriv(SoftwareOpenGL) $::UseLocalOpenGL
	    if { $::GidPriv(SoftwareOpenGL)} {
		WarnWinDirect [_ "Changing to software rendering."] .softogl
	    } else {
		WarnWinDirect [_ "Changing to accelerated OpenGL."] .softogl
	    }
	    # if { $::tcl_platform(os) == "Darwin"} {
	    after 1000 GiD_Set OGL_emulateFrontBuffer $::UseLocalOpenGL
	    # }
	}
	#  -bd 1
	destroy .softogl
	if { $::tcl_platform(os) == "Linux"} {
	    # write the SoftwareOpenGL and OGL_configured variables in the user's gidDefaults file
	    # together with the default preferences
	    set fileDefaults [ GiveGidDefaultsFile]
	    if { [ file exists $fileDefaults] } {
		# create a new gidDefaults from previous gidDefaults
		file copy -force $fileDefaults ${fileDefaults}~
		set fi [ open ${fileDefaults}~]
	    } else {
		# create a new gidDefaults from BestDefaults
		set fi [ open [ GiveBestGidDefaults]]
	    }
	    set fo [ open $fileDefaults w]
	    puts $fo "SoftwareOpenGL $::GidPriv(SoftwareOpenGL)"
	    puts $fo "OGL_configured 1"
	    while { ![ eof $fi]} {
		set line [ gets $fi]
		set key [ lindex $line 0]
		if { $key eq "SoftwareOpenGL"} {
		    continue
		}
		puts $fo $line
	    }
	    close $fi
	    close $fo

	    # WarnWinDirect "Preferences saved. Please start $::GidPriv(ProgName) again in order to use the new saved settings." \
	    # 	.softogl info
	    # exit
	    if { [ info exists env(INI_ARG_0)]} {
		set exec_prog $env(INI_ARG_0)
	    } else {
		set exec_prog [ file rootname $argv0]
	    }
	    # WarnWinDirect "$exec_prog 'argv: $argv'"
	    update
	    update idletasks
	    catch {
		exec $exec_prog {*}$argv
	    }
	    exit
	}
    }

    ttk::button .softogl.b2 -text Cancel -command "destroy .softogl"
    #  -bd 1
    grid .softogl.f.lbl -sticky w -padx 4 -pady 2
    grid .softogl.f.rd1 -sticky w -padx 4 -pady 2
    grid .softogl.f.rd1txt -sticky news -padx 24 -pady 0
    grid .softogl.f.rd2 -sticky w -padx 4 -pady 2
    grid .softogl.f.rd2txt -sticky news -padx 24 -pady 0
    grid .softogl.f.txt -sticky news -padx 8 -pady 2
    grid .softogl.f -sticky news -padx 4 -pady 2 -columnspan 2
    grid columnconfigure .softogl.f 0 -weight 1
    # grid rowconfigure .softogl.f 0 -weight 1
    # grid rowconfigure .softogl.f 1 -weight 1
    grid rowconfigure .softogl.f 2 -weight 1
    # grid rowconfigure .softogl.f 3 -weight 1
    grid rowconfigure .softogl.f 4 -weight 1
    grid rowconfigure .softogl.f 5 -weight 1
    
    grid .softogl.b1 .softogl.b2 -pady 4 -sticky s
    grid columnconfigure .softogl 0 -weight 1
    grid columnconfigure .softogl 1 -weight 1
    grid rowconfigure .softogl 0 -weight 1
    grid rowconfigure .softogl 1 -weight 1

    tkwait window .softogl
}

# WarnWinDirect [ info library]
# WarnWinDirect -$auto_path-

if { [info exists GidPriv(SplashWindow)] && [IsSplashWindowOnInDefaults] } {
    GiDSplash
}

# until here splash
###puts SetOptions
SetOptions

#set tcl_interactive 0
set tk_strictMotif 0
# precision in floating point operations
set tcl_precision 17


#source $GIDDEFAULTTCL/init.tcl

package require BWidget

package require tablelist_tile
package require dialogwin
package require supergrid
package require widgetsgid
#package require Chameleon

if { [ tk windowingsystem] eq "aqua"} {
    set ::acceleratorKey Command
    set ::acceleratorText Command
} else {
    set ::acceleratorKey Control
    set ::acceleratorText Ctrl
}


bind . <<ThemeChanged>> {
  set colordefaultframe [ttk::style lookup $ttk::currentTheme -background]
  if { $colordefaultframe != "" } {
    set color #
    foreach { colorcomponent } [winfo rgb . $colordefaultframe] {
      set colorcomponent [expr ($colorcomponent/257)+17]
      if { $colorcomponent > 255 } { set colorcomponent 255 }
      append color [format "%%02x" $colorcomponent]            
    }        
    ttk::style configure "LightBackground.TFrame" -background $color
  } else {
    ttk::style configure "LightBackground.TFrame" -background #fdfae9
  }
}



#eval font configure NormalFont [font configure TkTextFont] ;#usada en pruebas en DataWindows con tile??


# this is to avoid send commands
tk appname gid
#catch { rename send "" }

wm protocol .gid WM_DELETE_WINDOW {GiD_Process Mescape Quit}
ChangeWindowTitle UNNAMED
#wm title . GID

#set first with default values and after overwrite with defaults
ReadDefaultValues [GiveBestGidDefaults]

if { $AlternativeConfFile != "" && $AlternativeConfFile != "NONE" } {
    if { [file readable $AlternativeConfFile] } {
	ReadDefaultValues $AlternativeConfFile
    } else {
	after idle [list tk_dialogRAM "" [_ "Error window"] \
		        [_ "Window configuration file not found"] \
		        error 0 [_ "OK"]]
    }
} else {
    ReadDefaultValues [GiveGidDefaultsFile]
}

#set menus after read defaults, to set the appropiated language
SetAllMenusVariables
DWInitUserDataOptions

SetGUIAppearanceColor

if { ![info exists GidPriv(RealFormat,Default)] } {
    set GidPriv(RealFormat,Default) %.4g
}

wm minsize .gid 1 1

#set aa "exec xprop -root | grep SERVER_OVERLAY_VISUAL | grep -v CUT_BUFFER"

if { $tcl_platform(os) != "SunOS" } {
    set overlay_flag 1
} else {
    set overlay_flag 0
}

if { $overlay_flag && ![catch { eval $aa } bb] } {
    regexp {.*=[ ]*([^ ,]+).*} $bb trash GidPriv(SERVER_OVERLAY_VISUALS)
    option add *OverlayWindowN*Visual $GidPriv(SERVER_OVERLAY_VISUALS)
} else { set GidPriv(SERVER_OVERLAY_VISUALS) ""}

#proc grab { args } { }


#option add *Menu*overlay true
###puts "frame .gid.phantom"
frame .gid.phantom -class OverlayWindowN

if { [winfo cells .gid.phantom] < 6 } {
    option add *Menu*selectColor black
    option add *Menu*disabledForeground white
} else {
    option add *Menu*disabledForeground grey60
}
#destroy .phantom

#frame .phantom

menu .gid.phantom.display3dbutton -tearoff no

###puts Create3rdButtonMenu
Create3rdButtonMenu .gid.phantom.display3dbutton
if { $tcl_platform(platform) != "windows" } {
    wm colormapwindows .gid .gid.phantom
}

# InitialOpenGLConfiguration
# if { ( $tcl_platform(platform) == "windows") || ( $::tcl_platform(os) == "Darwin") } {
if { ![ info exists ::GidPriv(SoftwareOpenGL)] } {
    set ::GidPriv(SoftwareOpenGL) 0
}

if { [ info exists OpenGLConfig] } {
    InitialOpenGLConfiguration
}

if { $GidPriv(SoftwareOpenGL)} {
    set extra_flags "-softrender true"
    set alphaFlag false
}
# }


###puts WmGidGeom

WmGidGeom .gid $MainWindowGeom
#wm deiconify .gid
#wm withdraw .gid

if { [winfo exists .splash] } {
  wm deiconify .gid
  raise .splash .gid
}
#wm geometry . $MainWindowGeom
###puts "update idletasks"
update idletasks
#wm withdraw .
###puts "frame .gid.central"

if { ![info exists GidPriv(MenuType)] } {
    if { ( $tcl_platform(platform) == "windows") || ( $tcl_platform(os) == "Darwin") } {
	set GidPriv(MenuType) "native"
    } else {
	set GidPriv(MenuType) "generic"
    }
} else {
    if { $GidPriv(MenuType) == "native" &&  ($tcl_platform(platform) != "windows" && $tcl_platform(os) != "Darwin") } {
	#native menus not available for this platform
	set GidPriv(MenuType) "generic"
    }
}

if { $GidPriv(MenuType) == "generic" } {
    frame .gid.topm -bd 0
    set ::GiDTBLayout(.gid.topm) [list]
}

foreach _f_ {top left right rightb bottom bottomc} {
  set ::GiDTBLayout([frame .gid.$_f_ -bd 0]) [list]
}

if { $tcl_platform(platform) != "windows" } {
    frame .gid.central
} else {
    frame .gid.central -relief sunken -bd 0
}


if { $::GidPriv(SoftwareOpenGL)} {
    set extra_flags "-softrender true"
    set alphaFlag false
} else {
    set extra_flags ""
    set alphaFlag true
}

# para forzar la geometria que hemos leido
# si no se hace, se pierde
#wm geometry .gid [ lindex  $MainWindowGeom 1]

###puts togl
#-alpha true -> aunque sea lo correcto en windows con algunos drivers "no certificados"
#da problemas al hacer el c.fill
set bpp [ winfo screendepth .]
if { ( $bpp == 24) ||  ( $bpp == 32)} {
    set rgbFlags "-redsize 8 -greensize 8 -bluesize 8 -alphasize 8 "
    append rgbFlags "-alpha $alphaFlag "
} elseif { $bpp == 16} {
    set rgbFlags "-redsize 5 -greensize 6 -bluesize 5 -alphasize 8 "
# } elseif { $bpp == 15} {
#     set rgbFlags "-redsize 5 -greensize 5 -bluesize 5 -alphasize 8 "
# } elseif { $bpp == 8} {
#     set rgbFlags "-redsize 3 -greensize 3 -bluesize 2 "
} else {
    set rgbFlags ""
}
set depthBits [ winfo depth .]
#set rgbFlags "-redsize 8 -greensize 8 -bluesize 8 -alphasize 8 -depthsize 24 -stencilsize 0"
append rgbFlags " -depthsize $depthBits"

# if { $::GidPriv(Configuration) eq "debug"} {
#     WarnWinDirect "bpp: $bpp\ndepth: $depthBits"
#     WarnWinDirect "rgb: $rgbFlags\nextra: $extra_flags\nalpha: $alphaFlag"
# }

#first try with stereo, if not, then without
set old_extra_flags $extra_flags
append extra_flags " -stereo true"

set res [ catch { \
		      eval [ subst "togl .gid.central.s -rgba true -double true   \
	-depth true -ident GID -privatecmap false -overlay $overlay_flag $rgbFlags $extra_flags" ]
} err_msg]

# no hace falta, pues el togl se traga el flag si no hay stereo, devuelve in ctx mono.

# # sin stereo
# if { $res} {
#     # WarnWinDirect "Try 1: $err_msg"
#     set extra_flags $old_extra_flags
#     catch {
# 	destroy .gid.central.s
#     }
#     set res [ catch { \
# 			  eval [ subst "togl .gid.central.s -rgba true -double true   \
# 	-depth true -ident GID -privatecmap false -overlay $overlay_flag $rgbFlags $extra_flags" ]
#     } err_msg]
# }

# ni lo reintentamos, pues al cabo de unos segundos Tk no acepta/pasa eventos

# if { $res} {
#     if { $::GidPriv(Configuration) eq "debug"} {
#         WarnWinDirect "Try 1; $err_msg"
#     }
#     if { [ winfo exists .gid.central.s]} {
#         destroy .gid.central.s
#     }
#      -alpha $alphaFlag
#     # relajamos nuestros requerimientos:
#     set res [ catch { \
#                               eval [ subst "togl .gid.central.s -rgba true -double true  \
#                 -depth true -ident GID -privatecmap false -overlay 0 $extra_flags"]
#     } err_msg]
#     if { $res && $::GidPriv(Configuration) eq "debug"} {
#             WarnWinDirect "Try 2: $err_msg"
#     }
# }

if { $res} {
    WarnWinDirect "Couldn't create a graphical window with '$extra_flags': $err_msg"
    puts "Couldn't create a graphical window with '$extra_flags': $err_msg"
}

# if { $res} {
#     if { [ winfo exists .gid.central.s]} {
#         destroy .gid.central.s
#     }
#
#     set rgbFlags "-redsize 5 -greensize 6 -bluesize 5 -alphasize 0 -depthsize 24 -stencilsize 0"
#     set alphaFlag false
#     WarnWinDirect "$rgbFlags\n$extra_flags\n$alphaFlag"
#
#     set res [ catch { \
#                           eval [ subst "togl .gid.central.s -rgba true -alpha $alphaFlag -double true   \
#         -depth true -ident GID -privatecmap false -overlay $overlay_flag $rgbFlags $extra_flags" ]
#     } err_msg]
#
#     if { $res} {
#         if { $::GidPriv(Configuration) eq "debug"} {
#             WarnWinDirect $err_msg
#         }
#         if { [ winfo exists .gid.central.s]} {
#             destroy .gid.central.s
#         }
#         set res [ catch { \
#                               eval [ subst "togl .gid.central.s -rgba true -alpha $alphaFlag -double true  \
#             -depth true -ident GID -privatecmap false -overlay 0 $extra_flags"]
#         } err_msg]
#         if { $res && $::GidPriv(Configuration) eq "debug"} {
#             WarnWinDirect $err_msg
#         }
#     }
# }

if { [.gid.central.s disable windows] } {
  wm withdraw .gid
} else {
  wm deiconify .gid
}


# . conf -cursor "@$GIDDEFAULT/bitmaps/gid_mini red"
###puts bindings
bind .gid.central.s <1> {
    focus -force .gid.comm.e
    set GidPriv(PressingButton1) 1
    .gid.central.s button 1 %x %y
    after 300 [list catch {  unset GidPriv(PressingButton1) } ]
}
bind .gid.central.s <ButtonRelease-1> {
    if { [info exists GidPriv(PressingButton1)] } {
	.gid.central.s button -r -f 1 %x %y
	break
    }
    .gid.central.s button -r 1 %x %y
}

set botonCentral 2
set botonDerecho 3

if { $::tcl_platform(os) == "Darwin" } {
    # http://wiki.tcl.tk/12987 FAQ 7.7:
    # For historical reasons, MacMice buttons 2 and 3 refer to the right and middle buttons
    # respectively, which is indeed the opposite way round from Windows and *nix systems.
    set botonCentral 3
    set botonDerecho 2
}


bind .gid.central.s <$botonCentral> { .gid.central.s button 2 %x %y }
##### to handle mouse wheel
if { $GidPriv(GiDRuedas)} {
    proc MouseWheelEventToButtonEvent { increment } {
	if { $increment > 0 } {
	    .gid.central.s button 4 $increment 0
	} else {
	    .gid.central.s button 5 $increment 0
	}
    }
    catch {
	bind .gid.central.s <4> { .gid.central.s button 4 %x %y }
	bind .gid.central.s <5> { .gid.central.s button 5 %x %y }
    }
    bind .gid <MouseWheel> { MouseWheelEventToButtonEvent %D }
}
bind .gid.central.s <Motion> { .gid.central.s motion %x %y }
bind .gid.central.s <Leave> { .gid.central.s leave %x %y }
bind .gid.central.s <Enter> { .gid.central.s enter %x %y }
bind .gid <Unmap> "UnMapAllWindows %W"
bind .gid <Map> "MapAllWindows %W"
bind .gid <${::acceleratorKey}-a> SelectJoinOrNoJoin
#bind .gid <${::acceleratorKey}-h> SelectionOnlyVisibleOrAll

set mxbutwidth 5 ; set PostButStack ""

bind .gid.central.s <$botonDerecho> {
    #%W gidmakecurrent
    set x [expr [winfo rootx .gid.central.s]+%x+2]
    set y [expr [winfo rooty .gid.central.s]+%y]
    catch {
	GiD_PopupMenu .gid.phantom.display3dbutton $x $y
    }
}

#bind .gid.central.s <${::acceleratorKey}-ButtonRelease-1> "ContextualEntity::Activate %W %x %y; break"
bind .gid.central.s <${::acceleratorKey}-ButtonPress-1> "ContextualEntity::Activate %W %x %y; break"



bind Listbox <4> {
    %W yview scroll -3 units
}

bind Listbox <5> {
    %W yview scroll 3 units
}

bind Text <4> {
    %W yview scroll -3 units
}

bind Text <5> {
    %W yview scroll 3 units
}

#pack .gid.central.s -fill both -expand yes -side left
#pack .gid.central -fill both -expand yes
# top
if { $GidPriv(MenuType) == "native" } {
    #reserve the row 0 to handle swapping at runtime between native and generic like menus
    set row 1
} else {
  grid .gid.topm -row 0 -column 0 -columnspan 4 -sticky "snew"
  grid columnconfigure .gid.topm 0 -weight 1
  set row 1
}
grid .gid.top -row $row -column 0 -columnspan 4 -sticky "snew"
grid columnconfigure .gid.top 0 -weight 1

incr row

set crow $row
# left
grid .gid.left -row $row -column 0 -sticky "ns"
grid rowconfigure .gid.left 0 -weight 1
# central
grid .gid.central -row $row -column 1 -sticky "snew"
grid .gid.central.s -sticky "snew"
grid rowconfigure .gid.central 0 -weight 1
grid columnconfigure .gid.central 0 -weight 1
# right toolbars
grid .gid.right -row $row -column 2 -sticky "ns"
grid rowconfigure .gid.right 0 -weight 1
# right buttons
grid .gid.rightb -row $row -column 3 -sticky "ns"
grid rowconfigure .gid.rightb 0 -weight 1
incr row
# bottom
grid .gid.bottom -row $row -column 0 -columnspan 4 -sticky "snew"
grid columnconfigure .gid.bottom 0 -weight 1
incr row
grid .gid.bottomc -row $row -column 0 -columnspan 4 -sticky "snew"
grid columnconfigure .gid.bottomc 0 -weight 1

grid rowconfigure .gid $crow -weight 1
grid columnconfigure .gid 1 -weight 1

set WarnLineVar ""
set ActionsMenuVar ""
set ActionsMenuVarType "INTERP"
set EntryVar ""
set EntryVarHilit ""

#test to speech texts using Microsoft Windows Speech com object
set SPEECH_WARNLINE 0
if { $SPEECH_WARNLINE == 1 && $::tcl_platform(platform) == "windows" } {
    set ::GiD_Voice ""
    catch {
	package require tcom
	set ::GiD_Voice [::tcom::ref createobject Sapi.SpVoice]
    }
} else {
    set ::GiD_Voice ""
}

proc ChangeWarnLineVar { val } {
    global WarnLineVar uparrow vecToUseInVectors
    update idletasks
    if { [string index $val 0] == "\r" } {
	set WarnLineVar "[.gid.comm.fwarn.list get end] [string range $val 1 end]"
	.gid.comm.fwarn.list del end
    } else {
	set WarnLineVar $val
    }
    .gid.comm.fwarn.list insert end $WarnLineVar


    if { $::GiD_Voice != "" } {
	$::GiD_Voice Speak $val 1
    }
    set uparrow [.gid.comm.fwarn.list size]
    .gid.comm.fwarn.list yview [expr $uparrow-2]
    if { [info exists vecToUseInVectors] } { GetVarVecs $val }
}

proc procTraceVar { name elm op } {
    global uparrow vecToUseInVectors
    upvar $name x
    #puts "value for $name is $x"

    if { $name ==  "ActionsMenuVar" } {
	SetButtons
    }
    if { $name ==  "WarnLineVar" } {
	.gid.comm.fwarn.list insert end $x
	set uparrow [.gid.comm.fwarn.list size]
	.gid.comm.fwarn.list yview [expr $uparrow-2]
	if { [info exists vecToUseInVectors] } { GetVarVecs $x }
    }
    if { $name ==  "EntryVar"} { .gid.comm.e delete 0 end ; .gid.comm.e insert 0 $x }
    if { $name ==  "EntryVarHilit" } { .gid.comm.e select from 0 ; .gid.comm.e select to end }
}
trace variable ActionsMenuVar w procTraceVar
trace variable EntryVar w procTraceVar
trace variable EntryVarHilit w procTraceVar

#wm geometry . $MainWindowGeom
#wm deiconify .

#update idletasks
###puts "OpenOutsideWindows PRE"
# OpenOutsideWindows PRE
###puts CreateIcon
CreateIcon

image create photo gidquestionhead -data {
    R0lGODlhKAAoAPcAAAAAAP///+/vpu/vp+7upu3tpubmpeXlpd/fpO7use3tsNjYoe3tsdfX
    odPTn8jImcbGmMPDl8DAlb6+lLOzja+vi6ysiamph6ysiqmpiJmZfYqKc4aGcIODbnh4Zmtr
    XGdnWl1dU1BQSVNTTFJSS01NR0REQEJCPkdHQ0ZGQj4+Ozs7Ojg4N8/Pz8XFxcTExMPDw7Cw
    sK+vr62traenp6amppubm5mZmY+Pj46OjoyMjIODg4KCgnd3d3Z2dnV1dXFxcWpqamhoaGRk
    ZGJiYl5eXl1dXVxcXFtbW1VVVVRUVFNTU1JSUlFRUUxMTEtLS0pKSklJSUhISEdHR0ZGRkRE
    REJCQkFBQT8/Pzw8PDs7Ozo6Ojk5OTg4ODMzMzIyMjExMTAwMC8vLy4uLi0tLSwsLCsrKykp
    KSgoKCcnJyYmJiUlJSQkJCIiIiEhISAgIBsbGxoaGhgYGBUVFRQUFBMTExISEhERERAQEA4O
    Dg0NDQgICAcHBwYGBgUFBQQEBP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAIAALAAAAAAoACgAAAj/AAEJHEiwoMGD
    CBMqXMiwocOHECNKPAijBo0aMCY29MHmjx8/fPrwuRNnSAyNBpfYuXMHzRk0Zdy4sYPnY5sZ
    KAHRqHNnjJgsWn6SyTIGpps7f/480ZjDThknV7SE6fIhQ4QIGZSESRMTqR6JOeg8eXIlhYoG
    BAoIWEvgQIMNa8jM/JMH4gw6S5pcyVJhrd+/AgpEaKOljB0/bR6uYVLkCgoPDABLFiDBzBYy
    SIE0HOLEyBMtYCQTaID2LwEWWbbM7NMwjBAmQEMk8JugQZgyZUAAvpA6ixulC3ssCSJli5Y1
    fy2A0WL8S2S/DaRqGYOHz8IwQZD0RlFGQgMLJ1IY/09hQcFfCFmAavltQ6EXIVBSG88S5ksY
    5qApAFbAwbiYoHgokZAMVQgxhRZapGccglmgIEYD+yHwU4JibAFTQjcMt1dQCd5H34P7PfCT
    elmIkYYcCf3gRBBbtJhgekCFYcF+GiCoBRkMTmdHQjtQ0ViCYzC3xRhZOPeXAh0oCFQWZRh3
    BYoI2SBFEfKlZ6MWIsy2lgIOpBaUlwleEcUXCb0wxxEHejnGFul5caQVbC55pRROYKEQHU5E
    gSCObCaYAgsNHHAABkQFxWYZCWoxhRM+KFQFnS+mVwaYYpQRhpUtzsfmFU9A0YJCNtwB1ZJ9
    pjcGF0mIEeQW/32ZYBRIULbBkBxXSJFgkwiWAcYEBhhAgReXvmjjFUwcoQNDoV4xH35Z+PAc
    ZWoYR8ayTiDhhENYuJEormJYMIBfBIyQqJVTUuHCQ3q4saCCC3zrVwlMJriFFEwwcSxELE2K
    aBYr/DUACZHS+0SjEbUAxx1ljJGwCStIsMADXgy76Bc5oEQEH2+4kTCTYIYZ1As5AdKCFXjY
    IZPGGpeBhhti8BByQThUIUcdduxRRxdEnPvyzjz37PPPQL8cEAA7
}

if { [GiD_Info Project Debug] == "debug" } {

    bind .gid <${::acceleratorKey}-x><${::acceleratorKey}-d> DebugMesherPost
    bind .gid <${::acceleratorKey}-x><${::acceleratorKey}-l> { auto_reset }

}
bind all <Triple-Button-1> {grab release [grab current .gid] }

#it is used in tkfbox and by RamDebugger in package mode
# moved to GLWidget-OpenGL.cc because msgcat requires it and to avoid incompatibility conflicts
# if { $::tcl_platform(platform) eq "windows" } {
#     load [file join $::GIDDEFAULT tclreg11.dll] registry
# }
#if { $::tcl_platform(platform) eq "windows" } {
#    package require registry
#}

# OLE commands in windows
if { [info commands olednd] != "" } {
    #olednd is a deprecated internal GiD command, removed from 7.5.10b version
    olednd bindtarget .gid FileName <Drop> "LoadFileInGid"
} else {
    #if {$::tcl_platform(platform) eq "windows"} {
      catch { package require tkdnd } ;# only if it is compiled
	if { [info commands dnd] != "" } {
	    proc DropMultipleFiles { files } {
		foreach i $files {
		    LoadFileInGid $i
		}
	    }
	    #dnd bindtarget .gid FileName <Drop> "LoadFileInGid %D"
	    dnd bindtarget .gid text/uri-list <Drop> [ list after idle "DropMultipleFiles %D"]
	}
    #}
}


set ramdebuggerinside 1 ;#1 if is included with GiD, all packages are inside scripts
if { $ramdebuggerinside } {
    catch {
	package require commR ;#modification (commR) of tcllib comm package
	comm::register gid 1
    }
    bind all <F12> {
	catch { package require RamDebugger }
    }
} else {
    #donde esta RamDebuger y tcl?
    if { $tcl_platform(platform) == "windows" } {
	lappend auto_path "E:/Programas/ActiveTcl8.4.9.0/lib"
	set where_RamDebugger "C:/gid project/scripts/ramdebugger"
	lappend auto_path ~/mytcltk
	set where_ActiveTcl_lib {E:/Programas/ActiveTcl8.4.9.0/lib/tcl8.4}
    } else {
	set where_RamDebugger "/home/miguel/GID/stuff/RamDebugger"
	set where_ActiveTcl_lib ""
    }

    catch {
	# enter here the path where you have installed RamDebugger, subdirectory addons
	# for example: c:\Program files\RamDebugger\addons
	lappend auto_path [file join $where_RamDebugger addons]
	package require commR ;#modification (commR) of tcllib package
	comm::register gid 1
    }
    bind all <F12> {
	# enter here the path where you have installed RamDebugger
	# for example: c:\Program files\RamDebugger
	lappend auto_path $where_RamDebugger
	if  { $where_ActiveTcl_lib != ""} {
	    lappend auto_path $where_ActiveTcl_lib
	    lappend auto_path [file join $where_ActiveTcl_lib tcl8.4]
	}
	catch { package require RamDebugger }
    }
}


#toolbarmacros::Init #kike: comentado para evitar que se lea dos veces el contenido

#-----------------------------------------------Debug funcs---------------------------------

proc PrimitiveConsole {} {
    catch { destroy .gid.tt }
    toplevel .gid.tt
    pack [text .gid.tt.t]
    bind .gid.tt.t <Return> {
	set func [%W get "[%W index current] linestart" "[%W index current] lineend"]
	%W insert end \n[eval $func]
    }
}

proc SearchMenus { { w . } {init 1} } {
    global GidPriv
    if { $init } {
	set GidPriv(pp) ""
    }
    catch {
	set kk [$w cget -postcommand]
	if { [regexp {[a-zA-z]+} $kk] } {
	    append GidPriv(pp) "$w ---$kk---\n"
	}
    }
    foreach i [winfo children $w] {
	SearchMenus $i 0
    }
    if { $init } {
	return $GidPriv(pp)
    }
}

proc CheckPostCommands { {w . } } {

    catch {
	set cc [$w cget -postcommand]
	$w conf -postcommand "WarnWinText $w--$cc; $cc"
    }
    foreach i [winfo children $w] {
	CheckPostCommands $i
    }
}

#bind .gid <${::acceleratorKey}-x><${::acceleratorKey}-d> PrimitiveConsole


#---------------------------------------------------------------------------

# toplevel .ddd
# pack [button .ddd.b -text Try... -command LoadEventsFile]
# pack [label .ddd.l -width 100] -fill x -expand yes
# pack [button .ddd.b2 -text cont -command "set GidPriv(eventtry) 1"]

set GidPriv(NoCoordEvents) "<Unmap> <FocusIn> <FocusOut> <Destroy>"
set GidPriv(DetailEvents) "<Enter> <Leave> <FocusIn> <FocusOut>"
set GidPriv(StupidEvents) "<Configure> <Map> <Unmap> <Motion> <Enter> <Leave>"

proc LoadEventsFile {} {
    global GidPriv
    set fileid [open event-list.txt-save r]

    rename grab grab-old
    proc grab { args } { }

    while { ![eof $fileid] } {
	set aa [gets $fileid]
	if { [string index $aa 0] != "<" } { continue }

	set seq [lindex $aa 0]

	if { $seq == "<Map>" || $seq == "<Configure>"} { continue }
#        if { $seq == "<Enter>" || $seq == "<Leave>"} { continue }
	if { [lindex $aa 1] == "" } { break }

	set what "event generate [lindex $aa 1] [lindex $aa 0]"
	
	if { [lindex $aa 2] != "??" && \
		[lsearch $GidPriv(NoCoordEvents) $seq] == -1 } {
	    append what " -x [lindex $aa 2]"
	}
	if { [lindex $aa 3] != "??" && \
		[lsearch $GidPriv(NoCoordEvents) $seq] == -1  } {
	    append what " -y [lindex $aa 3]"
	}
	if { [lindex $aa 4] != "??" && [lindex $aa 4] > 3 } {
	    update
	    focus [lindex $aa 1]
	    append what " -keycode [lindex $aa 4]"
	}
	if { [lindex $aa 5] != "??"  && \
		[lsearch $GidPriv(DetailEvents) *$seq*] != -1 } {
	    append what " -detail  [lindex $aa 5]"
	}
	if { [lindex $aa 6] != "??" && \
		[lsearch $GidPriv(DetailEvents) *$seq*] != -1  } {
	    append what " -mode  [lindex $aa 6]"
	}
	if { [lindex $aa 7] != "??" && \
		[lsearch $GidPriv(NoCoordEvents) $seq] == -1} {
	    append what " -rootx [lindex $aa 7]"
	}
	if { [lindex $aa 8] != "??" && \
		[lsearch $GidPriv(NoCoordEvents) $seq] == -1 } {
	    append what " -rooty [lindex $aa 8]"
	}
	.ddd.l conf -text $what
	update
	if { [lsearch $GidPriv(StupidEvents) $seq] == -1 } {
	    after 1000
	}
	eval $what
    }
    close $fileid

    rename grab ""
    rename grab-old grab
}


proc SetBindingsForMouse { w ibutton cmd x y } {
    GiD_Process {*}$cmd
    $w button 1 $x $y
    set ::GidPriv(OldButtonRelease-$ibutton) [bind $w <ButtonRelease-$ibutton>]
    bind $w <ButtonRelease-$ibutton> "%W button -r 1 %x %y ;\
	    GiD_Process escape ;\
	    bind %W <ButtonRelease-$ibutton> \$::GidPriv(OldButtonRelease-$ibutton) ;\
	    break"
}

proc ShiftMouseWheelEventToButtonEvent { w increment } {
    # llamamos al trackball simplemente para que gid nos ponga las funciones de la ruedecita
    GiD_Process Rotate Trackball
    if { $increment > 0 } {
	$w button 4 $increment 0
    } else {
	$w button 5 $increment 0
    }
    GiD_Process Escape
}

proc ShiftibuttonEventToButtonEvent { w ibutton } {
    # llamamos al trackball simplemente para que gid nos ponga las funciones de la ruedecita
    GiD_Process Rotate Trackball
    $w button $ibutton %x %y
    GiD_Process Escape
}

proc SetButtonMeaning { w } {
    global GidPriv
    if { $GidPriv(ChangeButtonMeaning) == "0" } {
	foreach "mod cmd" [list Alt "DynamicPan" Shift "Zoom Dynamic" Alt-Shift "Rotate Trackball"] {
	    bind $w <$mod-ButtonPress-1> "[list SetBindingsForMouse $w 1 $cmd %x %y] ; break"
	}
    } else {

	set botonCentral 2
	set botonDerecho 3
	
	if { $::tcl_platform(os) == "Darwin" } {
	    # http://wiki.tcl.tk/12987 FAQ 7.7:
	    # For historical reasons, MacMice buttons 2 and 3 refer to the right and middle buttons
	    # respectively, which is indeed the opposite way round from Windows and *nix systems.
	    set botonCentral 3
	    set botonDerecho 2
	}

	#new bindings
	bind $w <Shift-1> "[list SetBindingsForMouse $w 1 {Rotate Trackball} %x %y] ; break"
	bind $w <Shift-$botonCentral> "[list SetBindingsForMouse $w 2 {Zoom Dynamic} %x %y] ; break"
	bind $w <Shift-$botonDerecho> "[list SetBindingsForMouse $w 3 DynamicPan %x %y] ; break"

	if { $GidPriv(GiDRuedas)} {
	    catch {
		bind $w <Shift-4> "[list ShiftibuttonEventToButtonEvent $w 4] ; break"
		bind $w <Shift-5> "[list ShiftibuttonEventToButtonEvent $w 5] ; break"
	    }
	    bind .gid <Shift-MouseWheel> "[list ShiftMouseWheelEventToButtonEvent $w %D] ; break"
	}
    }
}

SetButtonMeaning .gid.central.s

#encoding system "utf-8" ; not set to utf-8 because files like results, etc are not utf-8

#LoadGidAddons must be called after load Tk, to modify some bindings, etc
LoadGidAddons

###puts "update idletasks"
###update idletasks
###puts fin
#ramsan
#proc .central.s { args } {
#    return [eval [concat .gid.central.s $args]]
#}

after idle ChangeTopMenusAndWindows

# to avoid printing anything
return
